/*******************************************************************************
*    MultiButton 1.0.                                                          *
*    Verwaltung eines Domotikasystem mit O2-Joggler                            *
*    nach einer Idee von Leonardo Buffa <leonardo.buffa@gmail.com>             *
*    Copyright (C) 2013-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include <wx/txtstrm.h>
#include <wx/arrstr.h>
#include <wx/filefn.h>
#include <wx/stdpaths.h>
#include "MainFrame.h"
#include "lib/CurlReader.h"

MainFrame::MainFrame(wxWindow* parent, int width, int height, XML conf, wxString baseURL, wxString imageBaseURL, wxString cacheDir, wxString beepDevice)
:
wxFrame(parent, wxID_ANY, wxT(""), wxPoint(0, 70), wxSize(width, height))
{
  this->config = conf;
  this->baseURL = baseURL;
  this->imageBaseURL = imageBaseURL;
  this->cacheDir = cacheDir;
  this->beepDevice = beepDevice;

  this->background = NULL;
  this->exitButton = NULL;
  this->updateTimer = NULL;
  this->screenSaverTimer = NULL;
  this->canUpdate = this->canTimerUpdate = true;

  this->labels.clear();
  this->images.clear();
  this->buttons.clear();
  this->htmls.clear();
  this->toggleStatus.clear();
  this->htmlTimers.clear();
  this->unlockButtons.clear();

  this->isLocked = false;

  this->initFrame();

  this->Centre(wxHORIZONTAL);
}

MainFrame::~MainFrame()
{
  this->tmpConfig.clearXML();
  this->deleteFrame();
}

/**
 * Initialisiert das Fenster.
 * Alle Objekte werden instanziert
 */
void MainFrame::initFrame()
{
  wxBitmap bkg, img, img1;
  std::vector<Button>::iterator ib;
  std::vector<Label>::iterator il;
  std::vector<Image>::iterator ii;
  std::vector<BitmapButton*>::iterator ibb;
  std::vector<HTML>::iterator ih;
  std::vector<wxTimer*>::iterator it;
  int id;
  BitmapButton *tmp;
  wxStaticText *lab;
  wxHtmlWindow *html;
  wxStaticBitmap *image;
  wxString ret;
  bool value;
  wxFont *font;
  wxTimer *htmlTimer;
  wxString tmpBeep, base, tmpDir;
  wxStandardPaths p;

  tmpDir = p.GetTempDir();
  if(!tmpDir.EndsWith(wxT("/")))
    tmpDir = tmpDir + wxT("/");

  this->beepFile = wxT("");
  if(this->config.beepFile.Length() != 0)
  {
    if(this->config.beepFile.StartsWith(wxT("file://")))
      this->beepFile = this->config.beepFile;
    else
    {
      base = this->config.beepFile.Right(this->config.beepFile.length() - this->config.beepFile.rfind('/') - 1);
      if(this->cacheDir.Length() != 0)
        tmpBeep = this->cacheDir + base;
      else
        tmpBeep = tmpDir + base; // falls keine Cachedirectory eingerichtet worden ist
      if(!this->getFile(this->config.beepFile, tmpBeep))
      {
        printf("Cannot retrieve %s. Disabling beep...\n", (const char*) this->config.beepFile.mb_str());
        this->config.beepFile = wxT("");
      }
      else
        this->beepFile = tmpBeep;
    }
  }

  wxInitAllImageHandlers();

  this->SetTitle(this->config.title);

  if(this->config.background.Length() != 0)
  {
    bkg = wxBitmap(this->loadBitmap(this->config.background));
    this->background = new wxStaticBitmap(this, wxID_ANY,
        bkg,
        wxPoint(0, 0), wxDefaultSize, 0);
  }

  if(this->config.hasExitButton)
  {
    img = wxBitmap(this->loadBitmap(this->config.exitButton.image));
    this->exitButton = new BitmapButton(this, wxID_ANY,
        img,
        bkg,
        this->config.exitButton.label,
        wxPoint(this->config.exitButton.x, this->config.exitButton.y),
        wxSize(this->config.exitButton.width, this->config.exitButton.height),
        wxBitmap(), this->config.exitButton.foreground);
    if(this->config.exitButton.labelX != -1)
      this->exitButton->setLabelX(this->config.exitButton.labelX);
    if(this->config.exitButton.labelY != -1)
      this->exitButton->setLabelY(this->config.exitButton.labelY);
    this->exitButton->setFontSize(this->config.exitButton.fontSize);
    this->exitButton->setBold(this->config.exitButton.bold);
    this->exitButton->setItalic(this->config.exitButton.italic);
  }

  for(id = 10000, ib = this->config.buttons.begin(); ib != this->config.buttons.end(); ib++, id++)
  {
    if(!ib->isUnlock)
    {
      if(ib->checkurl.Length() != 0)
      {
        ret = this->getValueFromURL(ib->checkurl);
        if(ib->toggleImage.Length() != 0)
        {
          if(ret.compare(wxT("1")) == 0)
            value = true;
          else
            value = false;
        }
        this->toggleStatus.push_back(value);
      }
      else
      {
        this->toggleStatus.push_back(false);
        value = false;
      }
      img = wxBitmap(this->loadBitmap(ib->image));
      if(ib->toggleImage.Length() != 0)
        img1 = wxBitmap(this->loadBitmap(ib->toggleImage));
      else
        img1 = wxBitmap();
      tmp = new BitmapButton(this, id,
          img,
          bkg,
          ib->label,
          wxPoint(ib->x, ib->y),
          wxSize(ib->width, ib->height),
          img1,
          ib->foreground,
          value);
      if(ib->labelX != -1)
        tmp->setLabelX(ib->labelX);
      if(ib->labelY != -1)
        tmp->setLabelY(ib->labelY);
      tmp->setFontSize(ib->fontSize);
      tmp->setBold(ib->bold);
      tmp->setItalic(ib->italic);
      if(ib->longPressTime != -1)
        tmp->setLongPressTime(ib->longPressTime);
      this->buttons.push_back(tmp);
    }
  }

  for(ii = this->config.images.begin(); ii != this->config.images.end(); ii++)
  {
    bkg = wxBitmap(this->loadBitmap(ii->image));
    image = new wxStaticBitmap(this, wxID_ANY,
        bkg,
        wxPoint(ii->x, ii->y), wxDefaultSize, 0);
    this->images.push_back(image);
  }

  for(il = this->config.labels.begin(); il != this->config.labels.end(); il++)
  {
    lab = new wxStaticText(this, wxID_ANY,
                    il->label, wxPoint(il->x, il->y), wxDefaultSize, il->style);
    lab->Wrap(-1);
    font = new wxFont(il->fontSize, 72, 90, 92, false, wxEmptyString);
    if(il->bold)
      font->SetWeight(wxFONTWEIGHT_BOLD);
    if(il->italic)
      font->SetStyle(wxFONTSTYLE_ITALIC);
    lab->SetFont(*font);
    lab->SetForegroundColour(il->foreground);
    if(il->checkurl.Length() != 0)
    {
      ret = this->getValueFromURL(il->checkurl);
      lab->SetLabel(ret);
    }
    this->labels.push_back(lab);
    delete(font);
  }

  for(id = 20000, ih = this->config.htmls.begin(); ih != this->config.htmls.end(); ih++, id++)
  {
    html = new wxHtmlWindow(this, id,
                            wxPoint(ih->x, ih->y), wxSize(ih->width, ih->height));
    html->LoadPage(ih->url);
    this->config.htmls[id - 20000].objectID = id - 20000;
    this->htmls.push_back(html);
    if(ih->refreshMS != -1)
    {
// Nur ein Timer pro RefreshInterval!!
      for(value = false, it = this->htmlTimers.begin(); it != this->htmlTimers.end(); it++)
        if((*it)->GetInterval() == ih->refreshMS)
          value = true;
      if(!value)
      {
        htmlTimer = new wxTimer();
        htmlTimer->Connect(wxEVT_TIMER, wxTimerEventHandler(MainFrame::refreshURL), NULL, this);
        if(!htmlTimer->Start(ih->refreshMS))
          printf("Unable to start refresh\n");
        this->htmlTimers.push_back(htmlTimer);
      }
    }
  }

  if(this->config.updateInterval != -1)
  {
    this->updateTimer = new wxTimer();
    this->updateTimer->Connect(wxEVT_TIMER, wxTimerEventHandler(MainFrame::updateEvent), NULL, this);
    if(!this->updateTimer->Start(this->config.updateInterval * 1000, false))
      wxMessageBox(wxT("Unable to set update interval"), wxT("Multibutton"), wxICON_ERROR);
  }

  if(this->config.screenOffTime != -1)
  {
    this->screenSaverTimer = new wxTimer();
    this->screenSaverTimer->Connect(wxEVT_TIMER, wxTimerEventHandler(MainFrame::screenSaver), NULL, this);
  }

  for(ibb = this->buttons.begin(); ibb != this->buttons.end(); ibb++)
  {
    (*ibb)->Connect(wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainFrame::doGetClick), NULL, this);
  }
  if(this->config.hasExitButton)
    this->exitButton->Connect(wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainFrame::doClose), NULL, this);

  this->Layout();

  this->startScreensaverTimer();
}

/**
 * Startet, bzw. reinitialisiert den Timer für den Screensaver
 */
void MainFrame::startScreensaverTimer()
{
  if(this->config.screenOffTime != -1)
  {
    if(!this->screenSaverTimer->Start(this->config.screenOffTime * 1000, false))
      wxMessageBox(wxT("Unable to set screen saver"), wxT("Multibutton"), wxICON_ERROR);
  }
}

/**
 * Startet der Screensaver-Skript
 */
void MainFrame::screenSaver(wxTimerEvent& event)
{
  std::vector<Button>::iterator ib;
  std::vector<BitmapButton*>::iterator ibb;
  BitmapButton *tmp;
  wxBitmap bkg;

  wxExecute(this->config.screenOffCmd, wxEXEC_ASYNC | wxEXEC_NODISABLE);

  if(this->isLocked)
    return; // Damit es nicht nochmal den Unlock-Button einrichtet

  if(this->config.background.Length() != 0)
    bkg = wxBitmap(this->loadBitmap(this->config.background));

  for(ib = this->config.unlockButtons.begin(); ib != this->config.unlockButtons.end(); ib++)
  {
    tmp = new BitmapButton(this, wxID_ANY,
        wxBitmap(this->loadBitmap(ib->image)),
        bkg,
        ib->label,
        wxPoint(ib->x, ib->y),
        wxSize(ib->width, ib->height),
        wxBitmap(),
        ib->foreground,
        false);
    if(ib->labelX != -1)
      tmp->setLabelX(ib->labelX);
    if(ib->labelY != -1)
      tmp->setLabelY(ib->labelY);
    tmp->setFontSize(ib->fontSize);
    tmp->setBold(ib->bold);
    tmp->setItalic(ib->italic);
    this->unlockButtons.push_back(tmp);
    tmp->Connect(wxEVT_COMMAND_BUTTON_CLICKED , wxCommandEventHandler(MainFrame::doUnlock), NULL, this);
    this->isLocked = true;
  }
  if(this->isLocked)
  {
    if(this->config.hasExitButton)
      this->exitButton->Disconnect(wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainFrame::doClose), NULL, this);
    for(ibb = this->buttons.begin(); ibb != this->buttons.end(); ibb++)
      (*ibb)->Disconnect(wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainFrame::doGetClick), NULL, this);
  }
}

/**
 * Entsperrt den Display nach dem Screensaver
 * Diese Funktion startet einen Timer, damit es nicht
 * passieren kann, dass sie Objekte löscht, die
 * diese Funktion aufgerufen haben.
 */
void MainFrame::doUnlock(wxCommandEvent& event)
{
  this->timer = new wxTimer();
  this->timer->Connect(wxEVT_TIMER, wxTimerEventHandler(MainFrame::deleteUnlockButtons), NULL, this);
  if(!this->timer->Start(500, true))
    wxMessageBox(wxT("Unable to refresh window"), wxT("Multibutton"), wxICON_ERROR);
}

/**
 * Löscht alle "Unlock-Buttons" und verbindet
 * die doGetClick zu den anderen Buttons.
 * Diese Funktion wird mit einem Timer bei
 * der doUnlock() aufgerufen, ansonsten wird
 * das Objekt, das das Event gestartet hat, gelöscht
 * bevor die Callback-Funktion beendet ist, und das
 * Programm stürzt ab
 */
void MainFrame::deleteUnlockButtons(wxTimerEvent& event)
{
  std::vector<BitmapButton*>::iterator ib;
  std::vector<BitmapButton*>::iterator ibb;

  this->timer->Disconnect(wxEVT_TIMER, wxTimerEventHandler(MainFrame::deleteUnlockButtons), NULL, this);
  delete(this->timer);
  this->timer = NULL;

  for(ib = this->unlockButtons.begin(); ib != this->unlockButtons.end(); ib++)
  {
    (*ib)->Disconnect(wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainFrame::doUnlock), NULL, this);
    delete((*ib));
  }
  this->unlockButtons.clear();

  for(ibb = this->buttons.begin(); ibb != this->buttons.end(); ibb++)
    (*ibb)->Connect(wxEVT_COMMAND_BUTTON_CLICKED , wxCommandEventHandler(MainFrame::doGetClick), NULL, this);
  if(this->config.hasExitButton)
    this->exitButton->Connect(wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainFrame::doClose), NULL, this);

  this->isLocked = false;
  this->startScreensaverTimer();
}

/**
 * Callback-Funktion für den Timer.
 * Die URLs für die HTML-Objekten wird neugeladen
 *
 * Die Objekten werden anhand der TimerInterval identifiert,
 * denn es ist offensichtlich nicht möglich ein ID dem Timer
 * zu übergeben.
 * Deswegen wird auch nur EIN Timer pro Interval gesetzt
 */
void MainFrame::refreshURL(wxTimerEvent& event)
{
  std::vector<HTML>::iterator ih;
  std::vector<wxTimer*>::iterator iht;
  bool hasToRefresh;

  this->canTimerUpdate = this->canUpdate = false;

// Halte die Timers
  for(iht = this->htmlTimers.begin(); iht != this->htmlTimers.end(); iht++)
    (*iht)->Stop();

  for(ih = this->config.htmls.begin(); ih != this->config.htmls.end(); ih++)
  {
    hasToRefresh = false;
    if(this->isLocked)
    {
      if((*ih).alwaysRefresh)
        hasToRefresh = true;
    }
    else
      hasToRefresh = true;
    if(hasToRefresh && ih->refreshMS == event.GetInterval())
      this->htmls[ih->objectID]->LoadPage(ih->url);
  }

// Starte die Timers neu
  for(iht = this->htmlTimers.begin(); iht != this->htmlTimers.end(); iht++)
    (*iht)->Start();

  this->canTimerUpdate = this->canUpdate = true;
}

/**
 * Löscht alle Objekte, damit das neue
 * Fenster initialisiert werden kann.
 */
void MainFrame::deleteFrame()
{
  std::vector<BitmapButton*>::iterator ib;
  std::vector<wxStaticText*>::iterator il;
  std::vector<wxStaticBitmap*>::iterator ii;
  std::vector<wxHtmlWindow*>::iterator ih;
  std::vector<wxTimer*>::iterator iht;

  this->canTimerUpdate = this->canUpdate = false;
  if(this->screenSaverTimer != NULL)
  {
    this->screenSaverTimer->Disconnect(wxEVT_TIMER, wxTimerEventHandler(MainFrame::screenSaver), NULL, this);
    delete(this->screenSaverTimer);
    this->screenSaverTimer = NULL;
  }
  for(iht = this->htmlTimers.begin(); iht != this->htmlTimers.end(); iht++)
  {
    (*iht)->Disconnect(wxEVT_TIMER, wxTimerEventHandler(MainFrame::refreshURL), NULL, this);
    delete((*iht));
  }
  if(this->updateTimer != NULL)
  {
    this->updateTimer->Disconnect(wxEVT_TIMER, wxTimerEventHandler(MainFrame::updateEvent), NULL, this);
    delete(this->updateTimer);
    this->updateTimer = NULL;
  }
  if(this->background != NULL)
  {
    delete(this->background);
    this->background = NULL;
  }
  if(this->exitButton != NULL)
  {
    this->exitButton->Disconnect(wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainFrame::doClose), NULL, this);
    delete(this->exitButton);
    this->exitButton = NULL;
  }

  for(ib = this->buttons.begin(); ib != this->buttons.end(); ib++)
  {
    (*ib)->Disconnect(wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(MainFrame::doGetClick), NULL, this);
    delete((*ib));
  }
  for(il = this->labels.begin(); il != this->labels.end(); il++)
    delete((*il));
  for(ii = this->images.begin(); ii != this->images.end(); ii++)
    delete((*ii));
  for(ih = this->htmls.begin(); ih != this->htmls.end(); ih++)
    delete((*ih));

  this->buttons.clear();
  this->labels.clear();
  this->images.clear();
  this->htmls.clear();
  this->toggleStatus.clear();
  this->htmlTimers.clear();
  this->canTimerUpdate = this->canUpdate = true;
  this->config.clearXML();
}

/**
 * Callback-Funktion für die Knöpfe.
 * Je nachdem, was der Knopf für ein Typ ist (exitbutton,
 * togglebutton, normale button), wird eine URL aufgerufen
 */
void MainFrame::doGetClick(wxCommandEvent& event)
{
  Button btn = this->config.buttons[event.GetId() - 10000];
  wxString tmp;
  bool hasToRefresh = false;

// Display gesperrt, nichts machen
  if(this->isLocked)
    return;

  bool *data = (bool*) event.GetClientData();

  this->startScreensaverTimer();
  if(this->canUpdate)
  {
    this->canTimerUpdate = this->canUpdate = false;
    if(btn.beep && this->beepFile.Length() != 0)
      this->playBeep();
    if(*data && btn.longPress.Length() != 0)
    {
// Langer Klick
      tmp = wxT("");
      if(btn.longPress.compare(wxT("repeatlast")) == 0)
      {
        if(this->config.buttons[event.GetId() - 10000].lastURL.Length() != 0)
          tmp = this->config.buttons[event.GetId() - 10000].lastURL;
      }
      else
        tmp = btn.longPress;
      if(tmp.Length() != 0)
      {
        if(!this->callURL(tmp, btn.ignoreAnswer))
          wxMessageBox(wxT("Invalid answer from server"), wxT("Multibutton"), wxICON_EXCLAMATION);
      }
    }
    else
    {
// Normaler Klick
      if(btn.pageURL.Length() != 0)
      {
        tmp = this->config.normalizeURL(btn.pageURL, this->baseURL);
        this->config.buttons[event.GetId() - 10000].lastURL = tmp;
        if(tmp.compare(this->config.pageURL) != 0)
        {
          this->tmpConfig.clearXML();
          if(this->tmpConfig.loadFile(btn.pageURL, this->baseURL, this->imageBaseURL))
            hasToRefresh = true;
          else
            wxMessageBox(wxT("Unable to read XML"), wxT("Multibutton"), wxICON_ERROR);
        }
      }
      else if(btn.messageBox.Length() != 0)
      {
        (new wxMessageDialog(this, btn.messageBox, wxT(""), wxOK))->ShowModal();
      }
      else if(btn.hasToDisplayVersion)
      {
        tmp = wxString::Format(wxT("Multibutton Version %s\n\n"
                                    "Authors:\n\tLuca Bertoncello <lucabert@lucabert.de>\n"
                                    "\tLeonardo Buffa <leonardo.buffa@gmail.com>\n\n"
                                    "This program is available under the GPL-2.0 license\n"
                                    "and WITHOUT ANY WARRANTY.\n"), wxT(VERSION));
        (new wxMessageDialog(this, tmp, wxT(""), wxOK))->ShowModal();
      }
      else if(btn.url.Length() != 0)
      {
        this->config.buttons[event.GetId() - 10000].lastURL = btn.url;
        if(!this->callURL(btn.url, btn.ignoreAnswer))
          wxMessageBox(wxT("Invalid answer from server"), wxT("Multibutton"), wxICON_EXCLAMATION);
      }
      else if(btn.onurl.Length() != 0 && btn.offurl.Length() != 0)
      {
        if(this->toggleStatus[event.GetId() - 10000])
        {
          this->config.buttons[event.GetId() - 10000].lastURL = btn.offurl;
          if(!this->callURL(btn.offurl, btn.ignoreAnswer))
            wxMessageBox(wxT("Invalid answer from server"), wxT("Multibutton"), wxICON_EXCLAMATION);
        }
        else
        {
          this->config.buttons[event.GetId() - 10000].lastURL = btn.onurl;
          if(!this->callURL(btn.onurl, btn.ignoreAnswer))
            wxMessageBox(wxT("Invalid answer from server"), wxT("Multibutton"), wxICON_EXCLAMATION);
        }
        this->toggleStatus[event.GetId() - 10000] = !this->toggleStatus[event.GetId() - 10000];
      }
      else
        printf("Invalid button: %d\n", event.GetId());
    }
    if(btn.hasToUpdateObjects)
      this->updateAllObjects();
    if(hasToRefresh)
    {
      this->timer = new wxTimer();
      this->timer->Connect(wxEVT_TIMER, wxTimerEventHandler(MainFrame::redrawWindow), NULL, this);
      if(!this->timer->Start(100, true))
        wxMessageBox(wxT("Unable to refresh window"), wxT("Multibutton"), wxICON_ERROR);
    }
    else
      this->canTimerUpdate = this->canUpdate = true;
  }
}

/**
 * Startet das "Beep"-Skript
 */
void MainFrame::playBeep()
{
  FILE *fp, *fp1;
  int c;

  if(this->beepDevice.Length() != 0 && this->beepFile.length() != 0)
  { // Nur ausführen, wenn /dev/dsp UND Beep-Datei vorhanden sind!!
    fp = fopen((const char*) this->beepFile.mb_str(), "r");
    fp1 = fopen((const char*) this->beepDevice.mb_str(), "w");
    while(!feof(fp))
      if(putc(getc(fp), fp1) < 0)
        printf("Unable to play file\n");
    fclose(fp1);
    fclose(fp);
  }
}

/**
 * Callback-Funktion für den Timer.
 * Die Aktualisierung der Objekten wird
 * hier gestartet
 */
void MainFrame::updateEvent(wxTimerEvent& event)
{
  if(this->canTimerUpdate)
  {
    this->canUpdate = this->canTimerUpdate = false;
    this->updateAllObjects();
    this->canUpdate = this->canTimerUpdate = true;
  }
}

/**
 * Aktualisiert alle Objekte des Fensters
 */
void MainFrame::updateAllObjects()
{
  std::vector<Button>::iterator ib;
  std::vector<Label>::iterator il;
  std::vector<Event>::iterator ie;
  int id;
  wxString ret;
  bool value, hasToLoad, hasToRefresh;

  for(id = 0, ib = this->config.buttons.begin(); ib != this->config.buttons.end(); ib++, id++)
  {
    if(!ib->isUnlock)
    {
      hasToRefresh = false;
      if(this->isLocked)
      {
        if((*ib).alwaysRefresh)
          hasToRefresh = true;
      }
      else
        hasToRefresh = true;
      if((*ib).checkurl.Length() != 0 && hasToRefresh)
      {
        ret = this->getValueFromURL(ib->checkurl);
        if(ret.compare(wxT("1")) == 0)
          value = true;
        else
          value = false;
// Beim Anlegen ist die "id" nicht immer vergeben, denn unlockbuttons haben KEINEN
// und werden entsprechend NICHT ins vector hinzugefügt!
// id soll dann nur aktualisieren, wenn der Button tatsächlich hinzugefügt worden ist
        this->buttons[id]->setToggled(value);
        this->toggleStatus[id] = value;
        this->buttons[id]->Refresh();
      }
    }
  }

  for(id = 0, il = this->config.labels.begin(); il != this->config.labels.end(); il++, id++)
  {
    hasToRefresh = false;
    if(this->isLocked)
    {
      if((*il).alwaysRefresh)
        hasToRefresh = true;
    }
    else
      hasToRefresh = true;
    if((*il).checkurl.Length() != 0 && hasToRefresh)
    {
      ret = this->getValueFromURL(il->checkurl);
      this->labels[id]->SetLabel(ret);
    }
  }

  for(hasToLoad = value= false, ie = this->config.events.begin(); ie != this->config.events.end(); ie++)
  {
    hasToRefresh = false;
    if(this->isLocked)
    {
      if((*ie).alwaysRefresh)
        hasToRefresh = true;
    }
    else
      hasToRefresh = true;
    if(hasToRefresh)
    {
      ret = this->getValueFromURL(ie->checkURL);
      if(ie->checkType.compare(wxT("==")) == 0)
      {
        if(ret.compare(ie->checkValue) == 0)
          value = true;
      }
      else if(ie->checkType.compare(wxT("!=")) == 0)
      {
        if(ret.compare(ie->checkValue) != 0)
          value = true;
      }
      else if(ie->checkType.compare(wxT(">")) == 0)
      {
        if(wxAtoi(ret) > wxAtoi(ie->checkValue))
          value = true;
      }
      else if(ie->checkType.compare(wxT("<")) == 0)
      {
        if(wxAtoi(ret) < wxAtoi(ie->checkValue))
          value = true;
      }
      else if(ie->checkType.compare(wxT(">=")) == 0)
      {
        if(wxAtoi(ret) >= wxAtoi(ie->checkValue))
          value = true;
      }
      else if(ie->checkType.compare(wxT("<=")) == 0)
      {
        if(wxAtoi(ret) <= wxAtoi(ie->checkValue))
          value = true;
      }
      if(value)
      {
        if(ie->actionURL.Length() != 0)
        {
          this->callURL(ie->actionURL, true);
        }
        if(ie->pageURL.Length() != 0)
        {
          this->tmpConfig.clearXML();
          if(this->tmpConfig.loadFile(ie->pageURL, this->baseURL, this->imageBaseURL))
            hasToLoad = true;
          else
            wxMessageBox(wxT("Unable to read XML"), wxT("Multibutton"), wxICON_ERROR);
        }
      }
    }
  }
// Es ist notwendig, dass das Laden der neuen Seite NACH der Schleife
// passiert, ansonsten wird das Programm abstürzen, denn this->config
// wird neugeschrieben!
  if(hasToLoad)
  {
    this->timer = new wxTimer();
    this->timer->Connect(wxEVT_TIMER, wxTimerEventHandler(MainFrame::redrawWindow), NULL, this);
    if(!this->timer->Start(500, true))
      wxMessageBox(wxT("Unable to refresh window"), wxT("Multibutton"), wxICON_ERROR);
  }
}

/**
 * Lädt eine Bitmap aus einem Server, bzw.
 * von einer Datei
 *
 * @param wxString url                    Die URL des Bildes
 * @return wxImage                        Das Bild
 */
wxImage MainFrame::loadBitmap(wxString url)
{
  wxImage img;
  wxString suffix;
  long type = wxBITMAP_TYPE_ANY;
  wxInputStream *httpStream;
  wxString tmp;
  CurlReader *cReader;
  bool foundInCache = false;

  suffix = url.Right(url.length() - url.rfind('.') - 1).Lower();

  if(suffix.compare(wxT("jpg")) == 0 || suffix.compare(wxT("jpeg")) == 0)
    type = wxBITMAP_TYPE_JPEG;
  if(suffix.compare(wxT("bmp")) == 0)
    type = wxBITMAP_TYPE_BMP;
  if(suffix.compare(wxT("cur")) == 0)
    type = wxBITMAP_TYPE_CUR;
  if(suffix.compare(wxT("gif")) == 0)
    type = wxBITMAP_TYPE_GIF;
  if(suffix.compare(wxT("png")) == 0)
    type = wxBITMAP_TYPE_PNG;
  if(suffix.compare(wxT("tif")) == 0)
    type = wxBITMAP_TYPE_TIF;
  if(suffix.compare(wxT("xbm")) == 0)
    type = wxBITMAP_TYPE_XBM;
  if(suffix.compare(wxT("xpm")) == 0)
    type = wxBITMAP_TYPE_XPM;

  if(url.StartsWith(wxT("file://")))
  {
    if(wxFile::Exists(url.Mid(7)))
      img.LoadFile(url.Mid(7));
    else
    {
      printf("Unable to read file %s\n", (const char*) url.mb_str());
      img.Destroy();
    }
  }
  else
  {
    if(this->cacheDir.Length() != 0)
    {
  // Sucht die Datei
      tmp = url.Right(url.length() - url.rfind('/') - 1);
      if(wxFileExists(this->cacheDir + tmp))
      {
        img.LoadFile(this->cacheDir + tmp);
        foundInCache = true;
      }
    }

    if(!foundInCache)
    {
      cReader = new CurlReader(url);
      if(cReader->load())
      {
        httpStream = cReader->getFile();
        img.LoadFile(*httpStream, type);
        wxDELETE(httpStream);
        if(this->cacheDir.Length() != 0)
        {
// Datei in die Cache speichern
          tmp = url.Right(url.length() - url.rfind('/') - 1);
          img.SaveFile(this->cacheDir + tmp);
        }
      }
      else
      {
        printf("Unable to get file %s. Answer from server: %s\n", (const char*) url.mb_str(), (const char*)cReader->errors().mb_str());
        img.Destroy();
      }
      delete(cReader);
    }
  }

  return img;
}

/**
 * Ruft ein URL auf und interpretiert die Antwort.
 * Diese Funktion wird meistens bei Knöpfe benutzt.
 *
 * @param wxString url                    Die URL
 * @param bool ignoreAnswer               Wenn true, wird die Antwort vom Server ignoriert.
 *                                        Standard: false
 */
bool MainFrame::callURL(wxString url, bool ignoreAnswer)
{
  int l;
  wxInputStream *httpStream;
  wxString ret;
  wxArrayString out;
  wxString tmp;
  CurlReader *cReader;

  if(url.StartsWith(wxT("file://")))
  {
    if(wxExecute(url.Mid(7), out, wxEXEC_SYNC | wxEXEC_NODISABLE) == 0)
    {
      ret = wxT("");
      for(l = 0; l < out.Count(); l++)
        ret = ret + out.Item(l);
    }
    else
      printf("Unable to call %s\n", (const char*) url.mb_str());
  }
  else
  {
    cReader = new CurlReader(url);
    if(cReader->load())
    {
      httpStream = cReader->getFile();
      wxTextInputStream f(*httpStream);
      ret = f.ReadLine();
      wxDELETE(httpStream);
    }
    else
    {
      printf("Unable to get file %s. Answer from server: %s\n", (const char*) url.mb_str(), (const char*)cReader->errors().mb_str());
      return false;
    }
    delete(cReader);
  }

  if(ignoreAnswer)
    return true;

  if(ret.compare(wxT("OK")) != 0)
    return false;
  return true;
}

/**
 * Ruft ein URL auf und gibt die Antwort zurück.
 * Diese Funktion wird meistens bei Objekten benutzt
 * um Daten anzuzeigen.
 *
 * @param wxString url                    Die URL
 * @param wxString                        Die Antwort vom Server (Inhalt)
 */
wxString MainFrame::getValueFromURL(wxString url)
{
  int l;
  wxInputStream *httpStream;
  wxString ret;
  wxArrayString out;
  wxString tmp;
  CurlReader *cReader;

  if(url.StartsWith(wxT("file://")))
  {
    if(wxExecute(url.Mid(7), out, wxEXEC_SYNC | wxEXEC_NODISABLE) == 0)
    {
      ret = wxT("");
      for(l = 0; l < out.Count(); l++)
      {
        if(ret.Length() != 0)
          ret = ret + wxT("\n");
        ret = ret + out.Item(l);
      }
    }
    else
      printf("Unable to call %s\n", (const char*) url.mb_str());
  }
  else
  {
    cReader = new CurlReader(url);
    if(cReader->load())
    {
      httpStream = cReader->getFile();
      wxTextInputStream f(*httpStream);
      ret = wxT("");
      while(!httpStream->Eof())
      {
        if(ret.Length() != 0)
          ret = ret + wxT("\n");
        ret = ret + f.ReadLine();
      }
      wxDELETE(httpStream);
    }
    else
      printf("Unable to get file %s. Answer from server: %s\n", (const char*) url.mb_str(), (const char*)cReader->errors().mb_str());
    delete(cReader);
  }
  ret.Replace(wxT("\\n"), wxT("\n"), true);
  ret.Replace(wxT("\\t"), wxT("\t"), true);

  return ret;
}

/**
 * Aktualisiert das Fenster.
 * Diese Funktion wird mit einem Timer bei
 * der doGetClick() aufgerufen, ansonsten wird
 * das Objekt, das das Event gestartet hat, gelöscht
 * bevor die Callback-Funktion beendet ist, und das
 * Programm stürzt ab
 */
void MainFrame::redrawWindow(wxTimerEvent& event)
{
  this->timer->Disconnect(wxEVT_TIMER, wxTimerEventHandler(MainFrame::redrawWindow), NULL, this);
  delete(this->timer);
  this->timer = NULL;
  this->deleteFrame();
  this->config = this->tmpConfig;
  this->initFrame();
  this->canTimerUpdate = this->canUpdate = true;
}

/**
 * Beendet das Programm
 */
void MainFrame::doClose(wxCommandEvent& event)
{
// Display gesperrt, nichts machen
  if(this->isLocked)
    return;

  if(this->config.exitButton.beep && this->beepFile.Length() != 0)
    this->playBeep();
  this->Destroy();
}

/**
 * Holt eine Datei vom fernen Server und speichert sie
 * in die Cache.
 *
 * @param wxString url                    URL zur Datei zu laden
 * @param wxString localFile              Pfad zur lokalen Datei
 * @return bool                           true wenn alles in Ordnung, false ansonsten
 */
bool MainFrame::getFile(wxString url, wxString localFile)
{
  CurlReader *cReader;
  bool ret = false;
  wxFileOutputStream *fos;
  wxInputStream *is;

  cReader = new CurlReader(url);
  if(cReader->load())
  {
    fos = new wxFileOutputStream(localFile);
    if(fos->IsOk())
    {
      is = cReader->getFile();
      fos->Write(*is);
      fos->Close();
      wxDELETE(is);
      ret = true;
    }
    delete(fos);
  }
  delete(cReader);

  return ret;
}
