/*******************************************************************************
*    MultiButton 1.0.                                                          *
*    Verwaltung eines Domotikasystem mit O2-Joggler                            *
*    nach einer Idee von Leonardo Buffa <leonardo.buffa@gmail.com>             *
*    Copyright (C) 2013-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#ifndef MAINFRAME_H_
#define MAINFRAME_H_

#include <wx/wx.h>
#include <wx/url.h>
#include <wx/protocol/http.h>
#include <wx/wfstream.h>
#include <wx/html/htmlwin.h>
#include "lib/BitmapButton.h"
#include "lib/XML.h"

#define VERSION         "1.0-20130704"

class MainFrame : public wxFrame
{
public:
  MainFrame(wxWindow* parent, int width, int height, XML conf, wxString baseURL, wxString imageBaseURL, wxString cacheDir, wxString beepDevice);
  virtual
  ~MainFrame();

protected:
  void doGetClick(wxCommandEvent& event);
  void doUnlock(wxCommandEvent& event);
  void doClose(wxCommandEvent& event);
  wxImage loadBitmap(wxString url);
  void initFrame();
  void deleteFrame();
  void redrawWindow(wxTimerEvent& event);
  void updateEvent(wxTimerEvent& event);
  void refreshURL(wxTimerEvent& event);
  void screenSaver(wxTimerEvent& event);
  bool callURL(wxString url, bool ignoreAnswer = false);
  wxString getValueFromURL(wxString url);
  void updateAllObjects();
  void deleteUnlockButtons(wxTimerEvent& event);

private:
  void playBeep();
  void startScreensaverTimer();
  bool getFile(wxString url, wxString localFile);

  std::vector<wxStaticText*> labels;
  std::vector<wxStaticBitmap*> images;
  std::vector<BitmapButton*> buttons, unlockButtons;
  std::vector<wxHtmlWindow*> htmls;
  std::vector<wxTimer*> htmlTimers;
  std::vector<bool> toggleStatus;
  BitmapButton *exitButton;
  wxStaticBitmap *background;
  XML config, tmpConfig;
  wxTimer *timer, *updateTimer, *screenSaverTimer;
  wxString baseURL, imageBaseURL, cacheDir, beepFile, beepDevice;
  bool canUpdate, canTimerUpdate, isLocked;
};

#endif /* MAINFRAME_H_ */
