/*******************************************************************************
*    MultiButton 1.0.                                                          *
*    Verwaltung eines Domotikasystem mit O2-Joggler                            *
*    nach einer Idee von Leonardo Buffa <leonardo.buffa@gmail.com>             *
*    Copyright (C) 2013-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include <wx/url.h>
#include <wx/protocol/http.h>
#include <wx/wfstream.h>
#include "XML.h"
#include "CurlReader.h"

/**
 * Hilfsobjekt für die Knöpfe
 */
Button::Button(int x, int y, int width, int height,
               wxString image, wxString label, wxColour foreground,
               wxString toggleImage)
{
  this->x = x;
  this->y = y;
  this->width = width;
  this->height = height;
  this->image = image;
  this->label = label;
  this->foreground = foreground;
  this->toggleImage = toggleImage;
  this->url = wxT("");
  this->onurl = wxT("");
  this->offurl = wxT("");
  this->checkurl = wxT("");
  this->messageBox = wxT("");
  this->longPress = wxT("");
  this->labelX = this->labelY = -1;
  this->hasToUpdateObjects = this->hasToDisplayVersion = false;
  this->ignoreAnswer = true;
  this->fontSize = 9;
  this->bold = this->italic = false;
  this->beep = true;
  this->isUnlock = this->alwaysRefresh = false;
  this->longPressTime = -1;
}

Button::Button()
{
  this->x = this->y = this->width = this->height = -1;
  this->image = this->label = this->toggleImage = wxT("");
  this->foreground = wxNullColour;
  this->url = wxT("");
  this->onurl = wxT("");
  this->offurl = wxT("");
  this->checkurl = wxT("");
  this->messageBox = wxT("");
  this->longPress = wxT("");
  this->labelX = this->labelY = -1;
  this->hasToUpdateObjects = this->hasToDisplayVersion = false;
  this->ignoreAnswer = true;
  this->fontSize = 9;
  this->bold = this->italic = false;
  this->beep = true;
  this->isUnlock = this->alwaysRefresh = false;
  this->longPressTime = -1;
}

/**
 * Hilfobjekt für die Label
 */
Label::Label(int x, int y, long style, wxString label, wxColour foreground)
{
  this->x = x;
  this->y = y;
  this->style = style;
  this->label = label;
  this->foreground = foreground;
  this->fontSize = 18;
  this->bold = this->italic = false;
  this->alwaysRefresh = false;
}

/**
 * Hilfsobjekt für die Bilder
 */
Image::Image(int x, int y, wxString image)
{
  this->x = x;
  this->y = y;
  this->image = image;
}

/**
 * Hilfsobjekt für die HTML-Widgets
 */
HTML::HTML(int x, int y, int width, int height, wxString url, int refreshMS)
{
  this->x = x;
  this->y = y;
  this->width = width;
  this->height = height;
  this->url = url;
  this->refreshMS = refreshMS;
  this->objectID = -1;
  this->alwaysRefresh = false;
}

Event::Event(wxString checkURL, wxString checkValue, wxString checkType, wxString pageURL, wxString actionURL)
{
  this->checkURL = checkURL;
  this->checkValue = checkValue;
  this->checkType = checkType;
  this->pageURL = pageURL;
  this->actionURL = actionURL;
  this->alwaysRefresh = false;
}

XML::XML()
{
  this->hasExitButton = false;

  this->clearXML();
}

/**
 * Löscht die alte Daten.
 * Diese Method MUSS immer aufgerufen werden,
 * wenn eine neue Seite geladen wird.
 */
void XML::clearXML()
{
  this->buttons.clear();
  this->unlockButtons.clear();
  this->labels.clear();
  this->images.clear();
  this->htmls.clear();
  this->events.clear();

  this->hasExitButton = false;
  this->background = wxT("");
  this->title = wxT("");
  this->updateInterval = -1;
  this->screenOffCmd = wxT("");
  this->screenOffTime = -1;
  this->beepFile = wxT("");
}

/**
 * Lädt eine Konfigurationsdatei vom HTTP-Server oder
 * aus einer lokalen Datei
 *
 * @param wxString url                    Die URL der Konfigurationsdatei
 * @param wxString baseURL                Base für die URL, die in der Konfigurationsdatei
 *                                        gibt
 * @param wxString imageBaseURL           Directory, wo die Bilder standardmäßig liegen
 * @return bool                           true wenn die Datei richtig geparst werden konnte,
 *                                        false bei Fehler
 */
bool XML::loadFile(wxString url, wxString baseURL, wxString imageBaseURL)
{
  wxInputStream *httpStream;
  bool ret = true, conn = false;
  wxString file, tmp;
  CurlReader *cReader;

  if(!url.StartsWith(wxT("http://")) &&
     !url.StartsWith(wxT("https://")) &&
     !url.StartsWith(wxT("file://")))
    url = baseURL + url;

  this->pageURL = url;
  if(url.StartsWith(wxT("file://")))
  {
    if(wxFile::Exists(url.Mid(7)))
    {
      file = url.Mid(7);
      if(this->doc.Load(*file))
        ret = this->parseFile(baseURL, imageBaseURL);
      else
        ret = false;
    }
    else
    {
      printf("Unable to read file %s\n", (const char*) url.mb_str());
      ret = false;
    }
  }
  else
  {
    cReader = new CurlReader(url);
    if(cReader->load())
    {
      httpStream = cReader->getFile();
      if(this->doc.Load(*httpStream))
      {
        ret = this->parseFile(baseURL, imageBaseURL);
        wxDELETE(httpStream);
      }
      else
        ret = false;
    }
    else
    {
      printf("Unable to get file %s. Answer from server: %s\n", (const char*) url.mb_str(), (const char*)cReader->errors().mb_str());
      ret = false;
    }
    delete(cReader);
  }

  return ret;
}

/**
 * Parst eine gelesene Konfigurationsdatei
 *
 * @param wxString baseURL                Base für die URL, die in der Konfigurationsdatei
 *                                        gibt
 * @param wxString imageBaseURL           Directory, wo die Bilder standardmäßig liegen
 * @return bool                           true wenn die Datei richtig geparst werden konnte,
 *                                        false bei Fehler
 */
bool XML::parseFile(wxString baseURL, wxString imageBaseURL)
{
  wxXmlNode *child;
  wxColour tmpCol;
  wxString tmp;
  Button *tmpButton;
  Label *tmpLabel;
  HTML *tmpHTML;
  Image *tmpImage;
  Event *tmpEvent;
  bool ret = true;
  XML newXML;

// XML-Konfiguration parsen
  child = this->doc.GetRoot()->GetChildren();
  while(child)
  {
    if(child->GetName() == wxT("title"))
      this->title = child->GetNodeContent();
    else if(child->GetName() == wxT("background"))
      this->background = this->normalizeURL(child->GetNodeContent(), imageBaseURL);
    else if(child->GetName() == wxT("exitbutton"))
    {
      tmpCol.Set(child->GetPropVal(wxT("foreground"), wxT("#ffffff")));
      this->hasExitButton = true;

      this->exitButton = Button(wxAtoi(child->GetPropVal(wxT("x"), wxT("0"))),
                                wxAtoi(child->GetPropVal(wxT("y"), wxT("0"))),
                                wxAtoi(child->GetPropVal(wxT("width"), wxT("82"))),
                                wxAtoi(child->GetPropVal(wxT("height"), wxT("82"))),
                                this->normalizeURL(child->GetPropVal(wxT("background"), wxT("")), imageBaseURL),
                                this->normalizeLabel(child->GetPropVal(wxT("label"), wxT(""))),
                                tmpCol, wxT(""));
      tmp = child->GetPropVal(wxT("fontsize"), wxT(""));
      if(tmp.Length() != 0)
        this->exitButton.fontSize = wxAtoi(tmp);
      tmp = child->GetPropVal(wxT("beep"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          this->exitButton.beep = true;
        else
          this->exitButton.beep = false;
      }
      else
        this->exitButton.beep = true;
      tmp = child->GetPropVal(wxT("bold"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          this->exitButton.bold = true;
        else
          this->exitButton.bold = false;
      }
      else
        this->exitButton.bold = false;
      tmp = child->GetPropVal(wxT("italic"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          this->exitButton.italic = true;
        else
          this->exitButton.italic = false;
      }
      else
        this->exitButton.italic = false;
      tmp = child->GetPropVal(wxT("labelX"), wxT(""));
      if(tmp.Length() != 0)
        this->exitButton.labelX = wxAtoi(tmp);
      tmp = child->GetPropVal(wxT("labelY"), wxT(""));
      if(tmp.Length() != 0)
        this->exitButton.labelY = wxAtoi(tmp);
    }
    else if(child->GetName() == wxT("button"))
    {
      tmpCol.Set(child->GetPropVal(wxT("foreground"), wxT("#ffffff")));
      tmpButton = new Button(wxAtoi(child->GetPropVal(wxT("x"), wxT("0"))),
                             wxAtoi(child->GetPropVal(wxT("y"), wxT("0"))),
                             wxAtoi(child->GetPropVal(wxT("width"), wxT("82"))),
                             wxAtoi(child->GetPropVal(wxT("height"), wxT("82"))),
                             this->normalizeURL(child->GetPropVal(wxT("background"), wxT("")), imageBaseURL),
                             this->normalizeLabel(child->GetPropVal(wxT("label"), wxT(""))),
                             tmpCol, wxT(""));
      tmp = child->GetPropVal(wxT("fontsize"), wxT(""));
      if(tmp.Length() != 0)
        tmpButton->fontSize = wxAtoi(tmp);
      tmp = child->GetPropVal(wxT("beep"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpButton->beep = true;
        else
          tmpButton->beep = false;
      }
      else
        tmpButton->beep = true;
      tmp = child->GetPropVal(wxT("bold"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpButton->bold = true;
        else
          tmpButton->bold = false;
      }
      else
        tmpButton->bold = false;
      tmp = child->GetPropVal(wxT("italic"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpButton->italic = true;
        else
          tmpButton->italic = false;
      }
      else
        tmpButton->italic = false;
      tmp = child->GetPropVal(wxT("labelX"), wxT(""));
      if(tmp.Length() != 0)
        tmpButton->labelX = wxAtoi(tmp);
      tmp = child->GetPropVal(wxT("labelY"), wxT(""));
      if(tmp.Length() != 0)
        tmpButton->labelY = wxAtoi(tmp);
      tmp = child->GetPropVal(wxT("pagesource"), wxT(""));
      if(tmp.Length() != 0)
        tmpButton->pageURL = tmp;
      tmp = child->GetPropVal(wxT("url"), wxT(""));
      if(tmp.Length() != 0)
        tmpButton->url = this->normalizeURL(tmp, baseURL);
      tmp = child->GetPropVal(wxT("messagebox"), wxT(""));
      if(tmp.Length() != 0)
        tmpButton->messageBox = this->normalizeLabel(tmp);
      tmp = child->GetPropVal(wxT("hastoupdateobjects"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpButton->hasToUpdateObjects = true;
        else
          tmpButton->hasToUpdateObjects = false;
      }
      else
        tmpButton->hasToUpdateObjects = false;
      tmp = child->GetPropVal(wxT("ignoreanswer"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpButton->ignoreAnswer = true;
        else
          tmpButton->ignoreAnswer = false;
      }
      else
        tmpButton->ignoreAnswer = true;
      tmp = child->GetPropVal(wxT("displayversion"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpButton->hasToDisplayVersion = true;
        else
          tmpButton->hasToDisplayVersion = false;
      }
      else
        tmpButton->hasToDisplayVersion = false;
      tmp = child->GetPropVal(wxT("alwaysrefresh"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpButton->alwaysRefresh = true;
        else
          tmpButton->alwaysRefresh = false;
      }
      else
        tmpButton->alwaysRefresh = false;
      tmp = child->GetPropVal(wxT("longpress"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("repeatlast")) == 0)
          tmpButton->longPress = tmp;
        else
          tmpButton->longPress = this->normalizeURL(tmp, baseURL);
      }
      tmp = child->GetPropVal(wxT("longpresstime"), wxT(""));
      if(tmp.Length() != 0)
        tmpButton->longPressTime = wxAtol(tmp);
      this->buttons.push_back(*tmpButton);
      delete(tmpButton);
    }
    else if(child->GetName() == wxT("unlockbutton"))
    {
      tmpCol.Set(child->GetPropVal(wxT("foreground"), wxT("#ffffff")));
      tmpButton = new Button(wxAtoi(child->GetPropVal(wxT("x"), wxT("0"))),
                             wxAtoi(child->GetPropVal(wxT("y"), wxT("0"))),
                             wxAtoi(child->GetPropVal(wxT("width"), wxT("82"))),
                             wxAtoi(child->GetPropVal(wxT("height"), wxT("82"))),
                             this->normalizeURL(child->GetPropVal(wxT("background"), wxT("")), imageBaseURL),
                             this->normalizeLabel(child->GetPropVal(wxT("label"), wxT(""))),
                             tmpCol, wxT(""));
      tmpButton->isUnlock = true;
      tmp = child->GetPropVal(wxT("fontsize"), wxT(""));
      if(tmp.Length() != 0)
        tmpButton->fontSize = wxAtoi(tmp);
      tmp = child->GetPropVal(wxT("beep"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpButton->beep = true;
        else
          tmpButton->beep = false;
      }
      else
        tmpButton->beep = true;
      tmp = child->GetPropVal(wxT("bold"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpButton->bold = true;
        else
          tmpButton->bold = false;
      }
      else
        tmpButton->bold = false;
      tmp = child->GetPropVal(wxT("italic"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpButton->italic = true;
        else
          tmpButton->italic = false;
      }
      else
        tmpButton->italic = false;
      tmp = child->GetPropVal(wxT("labelX"), wxT(""));
      if(tmp.Length() != 0)
        tmpButton->labelX = wxAtoi(tmp);
      tmp = child->GetPropVal(wxT("labelY"), wxT(""));
      if(tmp.Length() != 0)
        tmpButton->labelY = wxAtoi(tmp);
      tmp = child->GetPropVal(wxT("pagesource"), wxT(""));
      if(tmp.Length() != 0)
        tmpButton->pageURL = tmp;
      tmp = child->GetPropVal(wxT("url"), wxT(""));
      if(tmp.Length() != 0)
        tmpButton->url = this->normalizeURL(tmp, baseURL);
      tmp = child->GetPropVal(wxT("messagebox"), wxT(""));
      if(tmp.Length() != 0)
        tmpButton->messageBox = this->normalizeLabel(tmp);
      tmp = child->GetPropVal(wxT("hastoupdateobjects"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpButton->hasToUpdateObjects = true;
        else
          tmpButton->hasToUpdateObjects = false;
      }
      else
        tmpButton->hasToUpdateObjects = false;
      tmp = child->GetPropVal(wxT("ignoreanswer"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpButton->ignoreAnswer = true;
        else
          tmpButton->ignoreAnswer = false;
      }
      else
        tmpButton->ignoreAnswer = true;
      tmp = child->GetPropVal(wxT("displayversion"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpButton->hasToDisplayVersion = true;
        else
          tmpButton->hasToDisplayVersion = false;
      }
      else
        tmpButton->hasToDisplayVersion = false;
      this->unlockButtons.push_back(*tmpButton);
      delete(tmpButton);
    }
    else if(child->GetName() == wxT("togglebutton"))
    {
      tmpCol.Set(child->GetPropVal(wxT("foreground"), wxT("#ffffff")));
      tmpButton = new Button(wxAtoi(child->GetPropVal(wxT("x"), wxT("0"))),
                             wxAtoi(child->GetPropVal(wxT("y"), wxT("0"))),
                             wxAtoi(child->GetPropVal(wxT("width"), wxT("82"))),
                             wxAtoi(child->GetPropVal(wxT("height"), wxT("82"))),
                             this->normalizeURL(child->GetPropVal(wxT("offbackground"), wxT("")), imageBaseURL),
                             this->normalizeLabel(child->GetPropVal(wxT("label"), wxT(""))),
                             tmpCol,
                             this->normalizeURL(child->GetPropVal(wxT("onbackground"), wxT("")), imageBaseURL));
      tmp = child->GetPropVal(wxT("fontsize"), wxT(""));
      if(tmp.Length() != 0)
        tmpButton->fontSize = wxAtoi(tmp);
      tmp = child->GetPropVal(wxT("beep"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpButton->beep = true;
        else
          tmpButton->beep = false;
      }
      else
        tmpButton->beep = true;
      tmp = child->GetPropVal(wxT("bold"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpButton->bold = true;
        else
          tmpButton->bold = false;
      }
      else
        tmpButton->bold = false;
      tmp = child->GetPropVal(wxT("italic"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpButton->italic = true;
        else
          tmpButton->italic = false;
      }
      else
        tmpButton->italic = false;
      tmp = child->GetPropVal(wxT("labelX"), wxT(""));
      if(tmp.Length() != 0)
        tmpButton->labelX = wxAtoi(tmp);
      tmp = child->GetPropVal(wxT("labelY"), wxT(""));
      if(tmp.Length() != 0)
        tmpButton->labelY = wxAtoi(tmp);
      tmp = child->GetPropVal(wxT("onurl"), wxT(""));
      if(tmp.Length() != 0)
        tmpButton->onurl = this->normalizeURL(tmp, baseURL);
      tmp = child->GetPropVal(wxT("offurl"), wxT(""));
      if(tmp.Length() != 0)
        tmpButton->offurl = this->normalizeURL(tmp, baseURL);
      tmp = child->GetPropVal(wxT("checkurl"), wxT(""));
      if(tmp.Length() != 0)
        tmpButton->checkurl = this->normalizeURL(tmp, baseURL);
      tmp = child->GetPropVal(wxT("hastoupdateobjects"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpButton->hasToUpdateObjects = true;
        else
          tmpButton->hasToUpdateObjects = false;
      }
      else
        tmpButton->hasToUpdateObjects = false;
      tmp = child->GetPropVal(wxT("ignoreanswer"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpButton->ignoreAnswer = true;
        else
          tmpButton->ignoreAnswer = false;
      }
      else
        tmpButton->ignoreAnswer = true;
      tmp = child->GetPropVal(wxT("alwaysrefresh"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpButton->alwaysRefresh = true;
        else
          tmpButton->alwaysRefresh = false;
      }
      else
        tmpButton->alwaysRefresh = false;
      tmp = child->GetPropVal(wxT("longpress"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("repeatlast")) == 0)
          tmpButton->longPress = tmp;
        else
          tmpButton->longPress = this->normalizeURL(tmp, baseURL);
      }
      tmp = child->GetPropVal(wxT("longpresstime"), wxT(""));
      if(tmp.Length() != 0)
        tmpButton->longPressTime = wxAtol(tmp);
      this->buttons.push_back(*tmpButton);
      delete(tmpButton);
    }
    else if(child->GetName() == wxT("label"))
    {
      tmpCol.Set(child->GetPropVal(wxT("foreground"), wxT("#ffffff")));
      tmpLabel = new Label(wxAtoi(child->GetPropVal(wxT("x"), wxT("0"))),
                           wxAtoi(child->GetPropVal(wxT("y"), wxT("0"))),
                           wxALIGN_CENTRE,
                           this->normalizeLabel(child->GetPropVal(wxT("label"), wxT(""))),
                           tmpCol);
      tmp = child->GetPropVal(wxT("checkurl"), wxT(""));
      if(tmp.Length() != 0)
        tmpLabel->checkurl = this->normalizeURL(tmp, baseURL);
      tmp = child->GetPropVal(wxT("fontsize"), wxT(""));
      if(tmp.Length() != 0)
        tmpLabel->fontSize = wxAtoi(tmp);
      tmp = child->GetPropVal(wxT("bold"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpLabel->bold = true;
        else
          tmpLabel->bold = false;
      }
      else
        tmpLabel->bold = false;
      tmp = child->GetPropVal(wxT("italic"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpLabel->italic = true;
        else
          tmpLabel->italic = false;
      }
      else
        tmpLabel->italic = false;
      tmp = child->GetPropVal(wxT("alwaysrefresh"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpLabel->alwaysRefresh = true;
        else
          tmpLabel->alwaysRefresh = false;
      }
      else
        tmpLabel->alwaysRefresh = false;
      this->labels.push_back(*tmpLabel);
      delete(tmpLabel);
    }
    else if(child->GetName() == wxT("image"))
    {
      tmpImage = new Image(wxAtoi(child->GetPropVal(wxT("x"), wxT("0"))),
                           wxAtoi(child->GetPropVal(wxT("y"), wxT("0"))),
                           this->normalizeURL(child->GetPropVal(wxT("url"), wxT("")), imageBaseURL));
      this->images.push_back(*tmpImage);
      delete(tmpImage);
    }
    else if(child->GetName() == wxT("web"))
    {
      tmpHTML = new HTML(wxAtoi(child->GetPropVal(wxT("x"), wxT("0"))),
                         wxAtoi(child->GetPropVal(wxT("y"), wxT("0"))),
                         wxAtoi(child->GetPropVal(wxT("width"), wxT("0"))),
                         wxAtoi(child->GetPropVal(wxT("height"), wxT("0"))),
                         this->normalizeURL(child->GetPropVal(wxT("url"), wxT("")), baseURL),
                         wxAtoi(child->GetPropVal(wxT("refreshms"), wxT("-1"))));
      tmp = child->GetPropVal(wxT("alwaysrefresh"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpHTML->alwaysRefresh = true;
        else
          tmpHTML->alwaysRefresh = false;
      }
      else
        tmpHTML->alwaysRefresh = false;
      this->htmls.push_back(*tmpHTML);
      delete(tmpHTML);
    }
    else if(child->GetName() == wxT("event"))
    {
      tmpEvent = new Event(this->normalizeURL(child->GetPropVal(wxT("checkurl"), wxT("")), baseURL),
                           child->GetPropVal(wxT("checkvalue"), wxT("")),
                           child->GetPropVal(wxT("checktype"), wxT("==")),
                           this->normalizeURL(child->GetPropVal(wxT("pageurl"), wxT("")), baseURL),
                           this->normalizeURL(child->GetPropVal(wxT("actionurl"), wxT("")), baseURL));
      tmp = child->GetPropVal(wxT("alwaysrefresh"), wxT(""));
      if(tmp.Length() != 0)
      {
        if(tmp.compare(wxT("true")) == 0)
          tmpEvent->alwaysRefresh = true;
        else
          tmpEvent->alwaysRefresh = false;
      }
      else
        tmpEvent->alwaysRefresh = false;
      this->events.push_back(*tmpEvent);
      delete(tmpEvent);
    }
    else if(child->GetName() == wxT("updateinterval"))
    {
      this->updateInterval = wxAtoi(child->GetNodeContent());
    }
    else if(child->GetName() == wxT("screensaver"))
    {
      this->screenOffTime = wxAtoi(child->GetPropVal(wxT("timeoutseconds"), wxT("")));
      this->screenOffCmd = child->GetPropVal(wxT("command"), wxT(""));
      if(!wxFileExists(this->screenOffCmd))
      {
        printf("File %s does not exists. Disabling screensaver...\n", (const char*) this->screenOffCmd.mb_str());
        this->screenOffCmd = wxT("");
        this->screenOffTime = -1;
      }
    }
    else if(child->GetName() == wxT("beepfile"))
    {
      this->beepFile = this->normalizeURL(child->GetNodeContent(), baseURL);
    }
    else if(child->GetName() == wxT("include"))
    {
      newXML.clearXML();
      if(newXML.loadFile(this->normalizeURL(child->GetNodeContent(), baseURL), baseURL, imageBaseURL))
        this->merge(newXML);
    }

    child = child->GetNext();
  }

  return ret;
}

/**
 * Normalisiert eine URL.
 * baseURL wird angehängt, wenn benötigt
 *
 * @param wxString url                    Die zu normalisieren URL
 * @param wxString baseURL                Base für die URL, die in der Konfigurationsdatei
 *                                        gibt
 * @return wxString                       Die normalisierte URL
 */
wxString XML::normalizeURL(wxString url, wxString baseURL)
{
  if(!url.StartsWith(wxT("http://")) &&
     !url.StartsWith(wxT("https://")) &&
     !url.StartsWith(wxT("file://")))
    return baseURL + url;
  return url;
}

/**
 * Normalisiert eine String.
 * \n wird interpretiert
 *
 * @param wxString label                  Die zu normalisieren String
 * @return wxString                       Die normalisierte String
 */
wxString XML::normalizeLabel(wxString label)
{
  wxString newLabel = label;

  newLabel.Replace(wxT("\\n"), wxT("\n"), true);

  return newLabel;
}

/**
 * Fügt die Inhalte der übergebenen Datei
 * der aktuellen Konfiguration hinzu
 *
 * @param XML doc                         Die Datei mit der zusätzlichen Konfiguration
 */
void XML::merge(XML doc)
{
  this->hasExitButton = doc.hasExitButton;
  this->screenOffCmd = doc.screenOffCmd;
  this->screenOffTime = doc.screenOffTime;
  this->beepFile = doc.beepFile;
  this->exitButton = doc.exitButton;
  this->background = doc.background;
  this->title = doc.title;
  this->updateInterval = doc.updateInterval;
  this->buttons.insert(this->buttons.end(), doc.buttons.begin(), doc.buttons.end());
  this->unlockButtons.insert(this->unlockButtons.end(), doc.unlockButtons.begin(), doc.unlockButtons.end());
  this->labels.insert(this->labels.end(), doc.labels.begin(), doc.labels.end());
  this->images.insert(this->images.end(), doc.images.begin(), doc.images.end());
  this->htmls.insert(this->htmls.end(), doc.htmls.begin(), doc.htmls.end());
  this->events.insert(this->events.end(), doc.events.begin(), doc.events.end());
}
