/*******************************************************************************
*    MultiButton 1.0.                                                          *
*    Verwaltung eines Domotikasystem mit O2-Joggler                            *
*    nach einer Idee von Leonardo Buffa <leonardo.buffa@gmail.com>             *
*    Copyright (C) 2013-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#ifndef XML_H_
#define XML_H_

#include <wx/wx.h>
#include <wx/xml/xml.h>
#include <vector>

class Button
{
public:
  Button(int x, int y, int width, int height,
             wxString image, wxString label, wxColour foreground,
             wxString toggleImage);
  Button();

  int x, y, width, height;
  wxString image, toggleImage, label, pageURL, messageBox;
  wxColour foreground;
  wxString url, onurl, offurl, checkurl, longPress;
  bool hasToUpdateObjects, ignoreAnswer, hasToDisplayVersion, beep, isUnlock, alwaysRefresh;
  int labelX, labelY, fontSize, bold, italic;
  wxString lastURL;
  long longPressTime;
};

class Label
{
public:
  Label(int x, int y, long style, wxString label, wxColour foreground);

  int x, y, fontSize, bold, italic;
  wxString label;
  long style;
  wxColour foreground;
  wxString checkurl;
  bool alwaysRefresh;
};

class Image
{
public:
  Image(int x, int y, wxString image);

  int x, y;
  wxString image;
};

class HTML
{
public:
  HTML(int x, int y, int width, int height, wxString url, int refreshMS);

  int x, y, width, height, refreshMS;
  wxString url;
  int objectID;
  bool alwaysRefresh;
};

class Event
{
public:
  Event(wxString checkURL, wxString checkValue, wxString checkType, wxString pageURL, wxString actionURL);

  wxString checkURL, checkValue, checkType, pageURL, actionURL;
  bool alwaysRefresh;
};

class XML
{
public:
  XML();
  bool loadFile(wxString url, wxString baseURL, wxString imageBaseURL);
  void clearXML();
  wxString normalizeURL(wxString url, wxString baseURL);

  wxString background, title;
  int updateInterval;
  std::vector<Button> buttons, unlockButtons;
  std::vector<Label> labels;
  std::vector<Image> images;
  std::vector<HTML> htmls;
  std::vector<Event> events;
  Button exitButton;
  wxString screenOffCmd;
  int screenOffTime;
  wxString beepFile;
  bool hasExitButton;
  wxString pageURL;

private:
  bool parseFile(wxString baseURL, wxString imageBaseURL);
  wxString normalizeLabel(wxString label);
  void merge(XML doc);
  wxXmlDocument doc;
};

#endif /* XML_H_ */
